@@ -1,17 +1,10 @@
 # -*- coding: utf-8 -*-
-# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4
+# vim: autoindent shiftwidth=4 expandtab textwidth=120 tabstop=4 softtabstop=4
 
 ###############################################################################
 # OpenLP - Open Source Lyrics Projection                                      #
 # --------------------------------------------------------------------------- #
-# Copyright (c) 2008-2013 Raoul Snyman                                        #
-# Portions copyright (c) 2008-2013 Tim Bentley, Gerald Britton, Jonathan      #
-# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
-# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
-# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
-# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
-# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
-# Frode Woldsund, Martin Zibricky                                             #
+# Copyright (c) 2008-2016 OpenLP Developers                                   #
 # --------------------------------------------------------------------------- #
 # This program is free software; you can redistribute it and/or modify it     #
 # under the terms of the GNU General Public License as published by the Free  #
@@ -26,30 +19,44 @@
 # with this program; if not, write to the Free Software Foundation, Inc., 59  #
 # Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
 ###############################################################################
+"""
+The :mod:`~openlp.core.lib.settingstab` module contains the base SettingsTab class which plugins use for adding their
+own tab to the settings dialog.
+"""
 
-from PyQt4 import QtGui
 
-class SettingsTab(QtGui.QWidget):
+from PyQt5 import QtWidgets
+
+
+from openlp.core.common import RegistryProperties
+
+
+class SettingsTab(QtWidgets.QWidget, RegistryProperties):
     """
-    SettingsTab is a helper widget for plugins to define Tabs for the settings
-    dialog.
+    SettingsTab is a helper widget for plugins to define Tabs for the settings dialog.
     """
     def __init__(self, parent, title, visible_title=None, icon_path=None):
         """
         Constructor to create the Settings tab item.
 
-        ``title``
-            The title of the tab, which is used internally for the tab handling.
-
-        ``visible_title``
-            The title of the tab, which is usually displayed on the tab.
+        :param parent:
+        :param title: The title of the tab, which is used internally for the tab handling.
+        :param visible_title: The title of the tab, which is usually displayed on the tab.
+        :param icon_path:
         """
-        QtGui.QWidget.__init__(self, parent)
-        self.tabTitle = title
-        self.tabTitleVisible = visible_title
-        self.settingsSection = self.tabTitle.lower()
+        super(SettingsTab, self).__init__(parent)
+        self.tab_title = title
+        self.tab_title_visible = visible_title
+        self.settings_section = self.tab_title.lower()
+        self.tab_visited = False
         if icon_path:
-            self.iconPath = icon_path
+            self.icon_path = icon_path
+        self._setup()
+
+    def _setup(self):
+        """
+        Run some initial setup. This method is separate from __init__ in order to mock it out in tests.
+        """
         self.setupUi()
         self.retranslateUi()
         self.initialise()
@@ -59,34 +66,32 @@ class SettingsTab(QtGui.QWidget):
         """
         Setup the tab's interface.
         """
-        self.tabLayout = QtGui.QHBoxLayout(self)
-        self.tabLayout.setObjectName(u'tabLayout')
-        self.leftColumn = QtGui.QWidget(self)
-        self.leftColumn.setObjectName(u'leftColumn')
-        self.leftLayout = QtGui.QVBoxLayout(self.leftColumn)
-        self.leftLayout.setMargin(0)
-        self.leftLayout.setObjectName(u'leftLayout')
-        self.tabLayout.addWidget(self.leftColumn)
-        self.rightColumn = QtGui.QWidget(self)
-        self.rightColumn.setObjectName(u'rightColumn')
-        self.rightLayout = QtGui.QVBoxLayout(self.rightColumn)
-        self.rightLayout.setMargin(0)
-        self.rightLayout.setObjectName(u'rightLayout')
-        self.tabLayout.addWidget(self.rightColumn)
+        self.tab_layout = QtWidgets.QHBoxLayout(self)
+        self.tab_layout.setObjectName('tab_layout')
+        self.left_column = QtWidgets.QWidget(self)
+        self.left_column.setObjectName('left_column')
+        self.left_layout = QtWidgets.QVBoxLayout(self.left_column)
+        self.left_layout.setContentsMargins(0, 0, 0, 0)
+        self.left_layout.setObjectName('left_layout')
+        self.tab_layout.addWidget(self.left_column)
+        self.right_column = QtWidgets.QWidget(self)
+        self.right_column.setObjectName('right_column')
+        self.right_layout = QtWidgets.QVBoxLayout(self.right_column)
+        self.right_layout.setContentsMargins(0, 0, 0, 0)
+        self.right_layout.setObjectName('right_layout')
+        self.tab_layout.addWidget(self.right_column)
 
     def resizeEvent(self, event=None):
         """
         Resize the sides in two equal halves if the layout allows this.
         """
         if event:
-            QtGui.QWidget.resizeEvent(self, event)
-        width = self.width() - self.tabLayout.spacing() - \
-            self.tabLayout.contentsMargins().left() - \
-            self.tabLayout.contentsMargins().right()
-        left_width = min(width - self.rightColumn.minimumSizeHint().width(),
-            width / 2)
-        left_width = max(left_width, self.leftColumn.minimumSizeHint().width())
-        self.leftColumn.setFixedWidth(left_width)
+            QtWidgets.QWidget.resizeEvent(self, event)
+        width = self.width() - self.tab_layout.spacing() - \
+            self.tab_layout.contentsMargins().left() - self.tab_layout.contentsMargins().right()
+        left_width = min(width - self.right_column.minimumSizeHint().width(), width // 2)
+        left_width = max(left_width, self.left_column.minimumSizeHint().width())
+        self.left_column.setFixedWidth(left_width)
 
     def retranslateUi(self):
         """
@@ -118,18 +123,16 @@ class SettingsTab(QtGui.QWidget):
         """
         self.load()
 
-    def postSetUp(self, postUpdate=False):
+    def post_set_up(self, post_update=False):
         """
         Changes which need to be made after setup of application
 
-        ``postUpdate``
-            Indicates if called before or after updates.
-
+        :param post_update: Indicates if called before or after updates.
         """
         pass
 
-    def tabVisible(self):
+    def tab_visible(self):
         """
         Tab has just been made visible to the user
         """
-        pass
+        self.tab_visited = True
