@@ -1,17 +1,10 @@
 # -*- coding: utf-8 -*-
-# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4
+# vim: autoindent shiftwidth=4 expandtab textwidth=120 tabstop=4 softtabstop=4
 
 ###############################################################################
 # OpenLP - Open Source Lyrics Projection                                      #
 # --------------------------------------------------------------------------- #
-# Copyright (c) 2008-2014 Raoul Snyman                                        #
-# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
-# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
-# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
-# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
-# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
-# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
-# Frode Woldsund, Martin Zibricky                                             #
+# Copyright (c) 2008-2016 OpenLP Developers                                   #
 # --------------------------------------------------------------------------- #
 # This program is free software; you can redistribute it and/or modify it     #
 # under the terms of the GNU General Public License as published by the Free  #
@@ -26,14 +19,19 @@
 # with this program; if not, write to the Free Software Foundation, Inc., 59  #
 # Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
 ###############################################################################
+"""
+The About dialog.
+"""
+import webbrowser
 
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtWidgets
 
-from aboutdialog import Ui_AboutDialog
 from openlp.core.lib import translate
 from openlp.core.utils import get_application_version
+from .aboutdialog import UiAboutDialog
 
-class AboutForm(QtGui.QDialog, Ui_AboutDialog):
+
+class AboutForm(QtWidgets.QDialog, UiAboutDialog):
     """
     The About dialog
     """
@@ -42,26 +40,27 @@ class AboutForm(QtGui.QDialog, Ui_AboutDialog):
         """
         Do some initialisation stuff
         """
-        QtGui.QDialog.__init__(self, parent)
-        applicationVersion = get_application_version()
-        self.setupUi(self)
-        about_text = self.aboutTextEdit.toPlainText()
-        about_text = about_text.replace(u'<version>',
-            applicationVersion[u'version'])
-        if applicationVersion[u'build']:
-            build_text = unicode(translate('OpenLP.AboutForm', ' build %s')) % \
-                applicationVersion[u'build']
-        else:
-            build_text = u''
-        about_text = about_text.replace(u'<revision>', build_text)
-        self.aboutTextEdit.setPlainText(about_text)
-        QtCore.QObject.connect(self.volunteerButton,
-            QtCore.SIGNAL(u'clicked()'), self.onVolunteerButtonClicked)
+        super(AboutForm, self).__init__(parent, QtCore.Qt.WindowSystemMenuHint | QtCore.Qt.WindowTitleHint)
+        self._setup()
 
-    def onVolunteerButtonClicked(self):
+    def _setup(self):
+        """
+        Set up the dialog. This method is mocked out in tests.
+        """
+        self.setup_ui(self)
+        application_version = get_application_version()
+        about_text = self.about_text_edit.toPlainText()
+        about_text = about_text.replace('<version>', application_version['version'])
+        if application_version['build']:
+            build_text = translate('OpenLP.AboutForm', ' build %s') % application_version['build']
+        else:
+            build_text = ''
+        about_text = about_text.replace('<revision>', build_text)
+        self.about_text_edit.setPlainText(about_text)
+        self.volunteer_button.clicked.connect(self.on_volunteer_button_clicked)
+
+    def on_volunteer_button_clicked(self):
         """
         Launch a web browser and go to the contribute page on the site.
         """
-        import webbrowser
-        url = u'http://openlp.org/en/contribute'
-        webbrowser.open_new(url)
+        webbrowser.open_new('http://openlp.org/en/contribute')
