@@ -1,17 +1,10 @@
 # -*- coding: utf-8 -*-
-# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4
+# vim: autoindent shiftwidth=4 expandtab textwidth=120 tabstop=4 softtabstop=4
 
 ###############################################################################
 # OpenLP - Open Source Lyrics Projection                                      #
 # --------------------------------------------------------------------------- #
-# Copyright (c) 2008-2013 Raoul Snyman                                        #
-# Portions copyright (c) 2008-2013 Tim Bentley, Gerald Britton, Jonathan      #
-# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
-# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
-# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
-# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
-# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
-# Frode Woldsund, Martin Zibricky                                             #
+# Copyright (c) 2008-2016 OpenLP Developers                                   #
 # --------------------------------------------------------------------------- #
 # This program is free software; you can redistribute it and/or modify it     #
 # under the terms of the GNU General Public License as published by the Free  #
@@ -27,8 +20,7 @@
 # Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
 ###############################################################################
 """
-The :mod:`cvsbible` modules provides a facility to import bibles from a set of
-CSV files.
+The :mod:`cvsbible` modules provides a facility to import bibles from a set of CSV files.
 
 The module expects two mandatory files containing the books and the verses.
 
@@ -55,93 +47,82 @@ There are two acceptable formats of the verses file.  They are:
         or
         "Genesis",1,2,"And the earth was without form, and void; and...."
 
-All CSV files are expected to use a comma (',') as the delimeter and double
-quotes ('"') as the quote symbol.
+All CSV files are expected to use a comma (',') as the delimiter and double quotes ('"') as the quote symbol.
 """
 import logging
 import chardet
 import csv
 
-from openlp.core.lib import Receiver, translate
+from openlp.core.common import translate
 from openlp.plugins.bibles.lib.db import BibleDB, BiblesResourcesDB
 
+
 log = logging.getLogger(__name__)
 
+
 class CSVBible(BibleDB):
     """
     This class provides a specialisation for importing of CSV Bibles.
     """
-    log.info(u'CSVBible loaded')
+    log.info('CSVBible loaded')
 
     def __init__(self, parent, **kwargs):
         """
-        Loads a Bible from a set of CSV files.
-        This class assumes the files contain all the information and
-        a clean bible is being loaded.
+        Loads a Bible from a set of CSV files. This class assumes the files contain all the information and a clean
+        bible is being loaded.
         """
         log.info(self.__class__.__name__)
         BibleDB.__init__(self, parent, **kwargs)
-        self.booksfile = kwargs[u'booksfile']
-        self.versesfile = kwargs[u'versefile']
+        self.books_file = kwargs['booksfile']
+        self.verses_file = kwargs['versefile']
 
     def do_import(self, bible_name=None):
         """
         Import the bible books and verses.
         """
-        self.wizard.progressBar.setValue(0)
-        self.wizard.progressBar.setMinimum(0)
-        self.wizard.progressBar.setMaximum(66)
+        self.wizard.progress_bar.setValue(0)
+        self.wizard.progress_bar.setMinimum(0)
+        self.wizard.progress_bar.setMaximum(66)
         success = True
         language_id = self.get_language(bible_name)
         if not language_id:
-            log.exception(u'Importing books from "%s" failed' % self.filename)
+            log.error('Importing books from "%s" failed' % self.filename)
             return False
         books_file = None
         book_list = {}
         # Populate the Tables
         try:
-            details = get_file_encoding(self.booksfile)
-            books_file = open(self.booksfile, 'r')
-            if not books_file.read(3) == '\xEF\xBB\xBF':
-                # no BOM was found
-                books_file.seek(0)
+            details = get_file_encoding(self.books_file)
+            books_file = open(self.books_file, 'r', encoding=details['encoding'])
             books_reader = csv.reader(books_file, delimiter=',', quotechar='"')
             for line in books_reader:
                 if self.stop_import_flag:
                     break
-                self.wizard.incrementProgressBar(unicode(
-                    translate('BiblesPlugin.CSVBible',
-                    'Importing books... %s')) %
-                    unicode(line[2], details['encoding']))
-                book_ref_id = self.get_book_ref_id_by_name(
-                    unicode(line[2], details['encoding']), 67, language_id)
+                self.wizard.increment_progress_bar(translate('BiblesPlugin.CSVBible', 'Importing books... %s')
+                                                   % line[2])
+                book_ref_id = self.get_book_ref_id_by_name(line[2], 67, language_id)
                 if not book_ref_id:
-                    log.exception(u'Importing books from "%s" '\
-                        'failed' % self.booksfile)
+                    log.error('Importing books from "%s" failed' % self.books_file)
                     return False
                 book_details = BiblesResourcesDB.get_book_by_id(book_ref_id)
-                self.create_book(unicode(line[2], details['encoding']),
-                    book_ref_id, book_details[u'testament_id'])
-                book_list[int(line[0])] = unicode(line[2], details['encoding'])
-            Receiver.send_message(u'openlp_process_events')
+                self.create_book(line[2], book_ref_id, book_details['testament_id'])
+                book_list.update({int(line[0]): line[2]})
+            self.application.process_events()
         except (IOError, IndexError):
-            log.exception(u'Loading books from file failed')
+            log.exception('Loading books from file failed')
             success = False
         finally:
             if books_file:
                 books_file.close()
         if self.stop_import_flag or not success:
             return False
-        self.wizard.progressBar.setValue(0)
-        self.wizard.progressBar.setMaximum(67)
+        self.wizard.progress_bar.setValue(0)
+        self.wizard.progress_bar.setMaximum(67)
         verse_file = None
         try:
             book_ptr = None
-            details = get_file_encoding(self.versesfile)
-            verse_file = open(self.versesfile, 'rb')
-            if not verse_file.read(3) == '\xEF\xBB\xBF':
-                # no BOM was found
-                verse_file.seek(0)
+            details = get_file_encoding(self.verses_file)
+            verse_file = open(self.verses_file, 'r', encoding=details['encoding'])
             verse_reader = csv.reader(verse_file, delimiter=',', quotechar='"')
             for line in verse_reader:
                 if self.stop_import_flag:
@@ -149,25 +130,21 @@ class CSVBible(BibleDB):
                 try:
                     line_book = book_list[int(line[0])]
                 except ValueError:
-                    line_book = unicode(line[0], details['encoding'])
+                    line_book = line[0]
                 if book_ptr != line_book:
                     book = self.get_book(line_book)
                     book_ptr = book.name
-                    self.wizard.incrementProgressBar(unicode(translate(
-                        'BiblesPlugin.CSVBible', 'Importing verses from %s...',
-                        'Importing verses from <book name>...')) % book.name)
+                    self.wizard.increment_progress_bar(
+                        translate('BiblesPlugin.CSVBible',
+                                  'Importing verses from %s...' % book.name, 'Importing verses from <book name>...'))
                     self.session.commit()
-                try:
-                    verse_text = unicode(line[3], details['encoding'])
-                except UnicodeError:
-                    verse_text = unicode(line[3], u'cp1252')
+                verse_text = line[3]
                 self.create_verse(book.id, line[1], line[2], verse_text)
-            self.wizard.incrementProgressBar(translate('BiblesPlugin.CSVBible',
-                'Importing verses... done.'))
-            Receiver.send_message(u'openlp_process_events')
+            self.wizard.increment_progress_bar(translate('BiblesPlugin.CSVBible', 'Importing verses... done.'))
+            self.application.process_events()
             self.session.commit()
         except IOError:
-            log.exception(u'Loading verses from file failed')
+            log.exception('Loading verses from file failed')
             success = False
         finally:
             if verse_file:
@@ -177,16 +154,17 @@ class CSVBible(BibleDB):
         else:
             return success
 
+
 def get_file_encoding(filename):
     """
     Utility function to get the file encoding.
     """
     detect_file = None
     try:
-        detect_file = open(filename, 'r')
+        detect_file = open(filename, 'rb')
         details = chardet.detect(detect_file.read(1024))
     except IOError:
-        log.exception(u'Error detecting file encoding')
+        log.exception('Error detecting file encoding')
     finally:
         if detect_file:
             detect_file.close()
