@@ -1,17 +1,10 @@
 # -*- coding: utf-8 -*-
-# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4
+# vim: autoindent shiftwidth=4 expandtab textwidth=120 tabstop=4 softtabstop=4
 
 ###############################################################################
 # OpenLP - Open Source Lyrics Projection                                      #
 # --------------------------------------------------------------------------- #
-# Copyright (c) 2008-2014 Raoul Snyman                                        #
-# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
-# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
-# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
-# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
-# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
-# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
-# Frode Woldsund, Martin Zibricky                                             #
+# Copyright (c) 2008-2016 OpenLP Developers                                   #
 # --------------------------------------------------------------------------- #
 # This program is free software; you can redistribute it and/or modify it     #
 # under the terms of the GNU General Public License as published by the Free  #
@@ -28,21 +21,25 @@
 ###############################################################################
 
 import logging
+import copy
 import os
 
-from PyQt4 import QtCore
+from PyQt5 import QtCore
 
-from openlp.core.lib import Receiver
+from openlp.core.common import Registry
 from openlp.core.ui import HideMode
+from openlp.core.lib import ServiceItemContext, ServiceItem
+from openlp.plugins.presentations.lib.pdfcontroller import PDF_CONTROLLER_FILETYPES
 
 log = logging.getLogger(__name__)
 
+
 class Controller(object):
     """
-    This is the Presentation listener who acts on events from the slide
-    controller and passes the messages on the the correct presentation handlers
+    This is the Presentation listener who acts on events from the slide controller and passes the messages on the the
+    correct presentation handlers.
     """
-    log.info(u'Controller loaded')
+    log.info('Controller loaded')
 
     def __init__(self, live):
         """
@@ -51,15 +48,14 @@ class Controller(object):
         self.is_live = live
         self.doc = None
         self.hide_mode = None
-        log.info(u'%s controller loaded' % live)
+        log.info('%s controller loaded' % live)
 
     def add_handler(self, controller, file, hide_mode, slide_no):
         """
-        Add a handler, which is an instance of a presentation and
-        slidecontroller combination. If the slidecontroller has a display
-        then load the presentation.
+        Add a handler, which is an instance of a presentation and slidecontroller combination. If the slidecontroller
+        has a display then load the presentation.
         """
-        log.debug(u'Live = %s, add_handler %s' % (self.is_live, file))
+        log.debug('Live = %s, add_handler %s' % (self.is_live, file))
         self.controller = controller
         if self.doc is not None:
             self.shutdown()
@@ -67,12 +63,14 @@ class Controller(object):
         if not self.doc.load_presentation():
             # Display error message to user
             # Inform slidecontroller that the action failed?
+            self.doc.slidenumber = 0
             return
         self.doc.slidenumber = slide_no
         self.hide_mode = hide_mode
+        log.debug('add_handler, slide_number: %d' % slide_no)
         if self.is_live:
             if hide_mode == HideMode.Screen:
-                Receiver.send_message(u'live_display_hide', HideMode.Screen)
+                Registry().execute('live_display_hide', HideMode.Screen)
                 self.stop()
             elif hide_mode == HideMode.Theme:
                 self.blank(hide_mode)
@@ -80,24 +78,23 @@ class Controller(object):
                 self.blank(hide_mode)
             else:
                 self.doc.start_presentation()
-                Receiver.send_message(u'live_display_hide', HideMode.Screen)
+                Registry().execute('live_display_hide', HideMode.Screen)
                 self.doc.slidenumber = 1
                 if slide_no > 1:
                     self.slide(slide_no)
 
     def activate(self):
         """
-        Active the presentation, and show it on the screen.
-        Use the last slide number.
+        Active the presentation, and show it on the screen. Use the last slide number.
         """
-        log.debug(u'Live = %s, activate' % self.is_live)
+        log.debug('Live = %s, activate' % self.is_live)
         if not self.doc:
             return False
         if self.doc.is_active():
             return True
         if not self.doc.is_loaded():
             if not self.doc.load_presentation():
-                log.warn(u'Failed to activate %s' % self.doc.filepath)
+                log.warning('Failed to activate %s' % self.doc.file_path)
                 return False
         if self.is_live:
             self.doc.start_presentation()
@@ -108,14 +105,14 @@ class Controller(object):
         if self.doc.is_active():
             return True
         else:
-            log.warn(u'Failed to activate %s' % self.doc.filepath)
+            log.warning('Failed to activate %s' % self.doc.file_path)
             return False
 
     def slide(self, slide):
         """
         Go to a specific slide
         """
-        log.debug(u'Live = %s, slide' % self.is_live)
+        log.debug('Live = %s, slide' % self.is_live)
         if not self.doc:
             return
         if not self.is_live:
@@ -131,9 +128,9 @@ class Controller(object):
 
     def first(self):
         """
-        Based on the handler passed at startup triggers the first slide
+        Based on the handler passed at startup triggers the first slide.
         """
-        log.debug(u'Live = %s, first' % self.is_live)
+        log.debug('Live = %s, first' % self.is_live)
         if not self.doc:
             return
         if not self.is_live:
@@ -149,9 +146,9 @@ class Controller(object):
 
     def last(self):
         """
-        Based on the handler passed at startup triggers the last slide
+        Based on the handler passed at startup triggers the last slide.
         """
-        log.debug(u'Live = %s, last' % self.is_live)
+        log.debug('Live = %s, last' % self.is_live)
         if not self.doc:
             return
         if not self.is_live:
@@ -167,9 +164,9 @@ class Controller(object):
 
     def next(self):
         """
-        Based on the handler passed at startup triggers the next slide event
+        Based on the handler passed at startup triggers the next slide event.
         """
-        log.debug(u'Live = %s, next' % self.is_live)
+        log.debug('Live = %s, next' % self.is_live)
         if not self.doc:
             return
         if not self.is_live:
@@ -178,14 +175,13 @@ class Controller(object):
             if not self.doc.is_active():
                 return
             if self.doc.slidenumber < self.doc.get_slide_count():
-                self.doc.slidenumber = self.doc.slidenumber + 1
+                self.doc.slidenumber += 1
                 self.poll()
             return
         if not self.activate():
             return
-        # The "End of slideshow" screen is after the last slide
-        # Note, we can't just stop on the last slide, since it may
-        # contain animations that need to be stepped through.
+        # The "End of slideshow" screen is after the last slide. Note, we can't just stop on the last slide, since it
+        # may contain animations that need to be stepped through.
         if self.doc.slidenumber > self.doc.get_slide_count():
             return
         self.doc.next_step()
@@ -193,9 +189,9 @@ class Controller(object):
 
     def previous(self):
         """
-        Based on the handler passed at startup triggers the previous slide event
+        Based on the handler passed at startup triggers the previous slide event.
         """
-        log.debug(u'Live = %s, previous' % self.is_live)
+        log.debug('Live = %s, previous' % self.is_live)
         if not self.doc:
             return
         if not self.is_live:
@@ -204,7 +200,7 @@ class Controller(object):
             if not self.doc.is_active():
                 return
             if self.doc.slidenumber > 1:
-                self.doc.slidenumber = self.doc.slidenumber - 1
+                self.doc.slidenumber -= 1
                 self.poll()
             return
         if not self.activate():
@@ -214,9 +210,9 @@ class Controller(object):
 
     def shutdown(self):
         """
-        Based on the handler passed at startup triggers slide show to shut down
+        Based on the handler passed at startup triggers slide show to shut down.
         """
-        log.debug(u'Live = %s, shutdown' % self.is_live)
+        log.debug('Live = %s, shutdown' % self.is_live)
         if not self.doc:
             return
         self.doc.close_presentation()
@@ -224,9 +220,9 @@ class Controller(object):
 
     def blank(self, hide_mode):
         """
-        Instruct the controller to blank the presentation
+        Instruct the controller to blank the presentation.
         """
-        log.debug(u'Live = %s, blank' % self.is_live)
+        log.debug('Live = %s, blank' % self.is_live)
         self.hide_mode = hide_mode
         if not self.doc:
             return
@@ -237,7 +233,7 @@ class Controller(object):
                 return
             if not self.doc.is_active():
                 return
-            Receiver.send_message(u'live_display_hide', HideMode.Theme)
+            Registry().execute('live_display_hide', HideMode.Theme)
         elif hide_mode == HideMode.Blank:
             if not self.activate():
                 return
@@ -245,9 +241,16 @@ class Controller(object):
 
     def stop(self):
         """
-        Instruct the controller to stop and hide the presentation
+        Instruct the controller to stop and hide the presentation.
         """
-        log.debug(u'Live = %s, stop' % self.is_live)
+        log.debug('Live = %s, stop' % self.is_live)
+        # The document has not been loaded yet, so don't do anything. This can happen when going live with a
+        # presentation while blanked to desktop.
+        if not self.doc:
+            return
+        # Save the current slide number to be able to return to this slide if the presentation is activated again.
+        if self.doc.is_active():
+            self.doc.slidenumber = self.doc.get_slide_number()
         self.hide_mode = HideMode.Screen
         if not self.doc:
             return
@@ -261,9 +264,9 @@ class Controller(object):
 
     def unblank(self):
         """
-        Instruct the controller to unblank the presentation
+        Instruct the controller to unblank the presentation.
         """
-        log.debug(u'Live = %s, unblank' % self.is_live)
+        log.debug('Live = %s, unblank' % self.is_live)
         self.hide_mode = None
         if not self.doc:
             return
@@ -271,11 +274,8 @@ class Controller(object):
             return
         if not self.activate():
             return
-        if self.doc.slidenumber and \
-            self.doc.slidenumber != self.doc.get_slide_number():
-            self.doc.goto_slide(self.doc.slidenumber)
         self.doc.unblank_screen()
-        Receiver.send_message(u'live_display_hide', HideMode.Screen)
+        Registry().execute('live_display_hide', HideMode.Screen)
 
     def poll(self):
         if not self.doc:
@@ -285,67 +285,99 @@ class Controller(object):
 
 class MessageListener(object):
     """
-    This is the Presentation listener who acts on events from the slide
-    controller and passes the messages on the the correct presentation handlers
+    This is the Presentation listener who acts on events from the slide controller and passes the messages on the
+    correct presentation handlers
     """
-    log.info(u'Message Listener loaded')
+    log.info('Message Listener loaded')
 
-    def __init__(self, mediaitem):
-        self.controllers = mediaitem.controllers
-        self.mediaitem = mediaitem
+    def __init__(self, media_item):
+        self._setup(media_item)
+
+    def _setup(self, media_item):
+        """
+        Start up code moved out to make mocking easier
+        :param media_item: The plugin media item handing Presentations
+        """
+        self.controllers = media_item.controllers
+        self.media_item = media_item
         self.preview_handler = Controller(False)
         self.live_handler = Controller(True)
         # messages are sent from core.ui.slidecontroller
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_start'), self.startup)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_stop'), self.shutdown)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_hide'), self.hide)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_first'), self.first)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_previous'), self.previous)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_next'), self.next)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_last'), self.last)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_slide'), self.slide)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_blank'), self.blank)
-        QtCore.QObject.connect(Receiver.get_receiver(),
-            QtCore.SIGNAL(u'presentations_unblank'), self.unblank)
+        Registry().register_function('presentations_start', self.startup)
+        Registry().register_function('presentations_stop', self.shutdown)
+        Registry().register_function('presentations_hide', self.hide)
+        Registry().register_function('presentations_first', self.first)
+        Registry().register_function('presentations_previous', self.previous)
+        Registry().register_function('presentations_next', self.next)
+        Registry().register_function('presentations_last', self.last)
+        Registry().register_function('presentations_slide', self.slide)
+        Registry().register_function('presentations_blank', self.blank)
+        Registry().register_function('presentations_unblank', self.unblank)
         self.timer = QtCore.QTimer()
         self.timer.setInterval(500)
-        QtCore.QObject.connect(
-            self.timer, QtCore.SIGNAL(u'timeout()'), self.timeout)
+        self.timer.timeout.connect(self.timeout)
 
     def startup(self, message):
         """
-        Start of new presentation
-        Save the handler as any new presentations start here
+        Start of new presentation. Save the handler as any new presentations start here
         """
+        log.debug('Startup called with message %s' % message)
         is_live = message[1]
         item = message[0]
-        log.debug(u'Startup called with message %s' % message)
         hide_mode = message[2]
         file = item.get_frame_path()
-        self.handler = item.title
-        if self.handler == self.mediaitem.Automatic:
-            self.handler = self.mediaitem.findControllerByType(file)
-            if not self.handler:
-                return
+        self.handler = item.processor
+        # When starting presentation from the servicemanager we convert
+        # PDF/XPS/OXPS-serviceitems into image-serviceitems. When started from the mediamanager
+        # the conversion has already been done at this point.
+        file_type = os.path.splitext(file.lower())[1][1:]
+        if file_type in PDF_CONTROLLER_FILETYPES:
+            log.debug('Converting from pdf/xps/oxps to images for serviceitem with file %s', file)
+            # Create a copy of the original item, and then clear the original item so it can be filled with images
+            item_cpy = copy.copy(item)
+            item.__init__(None)
+            if is_live:
+                self.media_item.generate_slide_data(item, item_cpy, False, False, ServiceItemContext.Live, file)
+            else:
+                self.media_item.generate_slide_data(item, item_cpy, False, False, ServiceItemContext.Preview, file)
+            # Some of the original serviceitem attributes is needed in the new serviceitem
+            item.footer = item_cpy.footer
+            item.from_service = item_cpy.from_service
+            item.iconic_representation = item_cpy.iconic_representation
+            item.image_border = item_cpy.image_border
+            item.main = item_cpy.main
+            item.theme_data = item_cpy.theme_data
+            # When presenting PDF/XPS/OXPS, we are using the image presentation code,
+            # so handler & processor is set to None, and we skip adding the handler.
+            self.handler = None
+        else:
+            if self.handler == self.media_item.automatic:
+                self.handler = self.media_item.find_controller_by_type(file)
+                if not self.handler:
+                    return
+            else:
+                # the saved handler is not present so need to use one based on file suffix.
+                if not self.controllers[self.handler].available:
+                    self.handler = self.media_item.find_controller_by_type(file)
+                    if not self.handler:
+                        return
         if is_live:
             controller = self.live_handler
         else:
             controller = self.preview_handler
-        controller.add_handler(self.controllers[self.handler], file, hide_mode,
-            message[3])
+        # When presenting PDF/XPS/OXPS, we are using the image presentation code,
+        # so handler & processor is set to None, and we skip adding the handler.
+        if self.handler is None:
+            self.controller = controller
+        else:
+            controller.add_handler(self.controllers[self.handler], file, hide_mode, message[3])
+            self.timer.start()
 
     def slide(self, message):
         """
-        React to the message to move to a specific slide
+        React to the message to move to a specific slide.
+
+        :param message: The message {1} is_live {2} slide
         """
         is_live = message[1]
         slide = message[2]
@@ -356,7 +388,9 @@ class MessageListener(object):
 
     def first(self, message):
         """
-        React to the message to move to the first slide
+        React to the message to move to the first slide.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
@@ -366,7 +400,9 @@ class MessageListener(object):
 
     def last(self, message):
         """
-        React to the message to move to the last slide
+        React to the message to move to the last slide.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
@@ -376,7 +412,9 @@ class MessageListener(object):
 
     def next(self, message):
         """
-        React to the message to move to the next animation/slide
+        React to the message to move to the next animation/slide.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
@@ -386,7 +424,9 @@ class MessageListener(object):
 
     def previous(self, message):
         """
-        React to the message to move to the previous animation/slide
+        React to the message to move to the previous animation/slide.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
@@ -396,18 +436,22 @@ class MessageListener(object):
 
     def shutdown(self, message):
         """
-        React to message to shutdown the presentation. I.e. end the show
-        and close the file
+        React to message to shutdown the presentation. I.e. end the show and close the file.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
             self.live_handler.shutdown()
+            self.timer.stop()
         else:
             self.preview_handler.shutdown()
 
     def hide(self, message):
         """
-        React to the message to show the desktop
+        React to the message to show the desktop.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
@@ -415,7 +459,9 @@ class MessageListener(object):
 
     def blank(self, message):
         """
-        React to the message to blank the display
+        React to the message to blank the display.
+
+        :param message: The message {1} is_live {2} slide
         """
         is_live = message[1]
         hide_mode = message[2]
@@ -424,7 +470,9 @@ class MessageListener(object):
 
     def unblank(self, message):
         """
-        React to the message to unblank the display
+        React to the message to unblank the display.
+
+        :param message: The message {1} is_live
         """
         is_live = message[1]
         if is_live:
@@ -432,9 +480,7 @@ class MessageListener(object):
 
     def timeout(self):
         """
-        The presentation may be timed or might be controlled by the
-        application directly, rather than through OpenLP. Poll occassionally
-        to check which slide is currently displayed so the slidecontroller
-        view can be updated
+        The presentation may be timed or might be controlled by the application directly, rather than through OpenLP.
+        Poll occasionally to check which slide is currently displayed so the slidecontroller view can be updated.
         """
         self.live_handler.poll()
