@@ -1,17 +1,10 @@
 # -*- coding: utf-8 -*-
-# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4
+# vim: autoindent shiftwidth=4 expandtab textwidth=120 tabstop=4 softtabstop=4
 
 ###############################################################################
 # OpenLP - Open Source Lyrics Projection                                      #
 # --------------------------------------------------------------------------- #
-# Copyright (c) 2008-2013 Raoul Snyman                                        #
-# Portions copyright (c) 2008-2013 Tim Bentley, Gerald Britton, Jonathan      #
-# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
-# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
-# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
-# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
-# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
-# Frode Woldsund, Martin Zibricky                                             #
+# Copyright (c) 2008-2016 OpenLP Developers                                   #
 # --------------------------------------------------------------------------- #
 # This program is free software; you can redistribute it and/or modify it     #
 # under the terms of the GNU General Public License as published by the Free  #
@@ -28,91 +21,92 @@
 ###############################################################################
 
 import sys
-from PyQt4 import QtGui, QtCore
+from PyQt5 import QtWidgets
 from ctypes import *
 from ctypes.wintypes import RECT
 
-class PPTViewer(QtGui.QWidget):
+
+class PPTViewer(QtWidgets.QWidget):
     """
     Standalone Test Harness for the pptviewlib library
     """
     def __init__(self, parent=None):
-        QtGui.QWidget.__init__(self, parent)
+        super(PPTViewer, self).__init__(parent)
         self.pptid = -1
-        self.setWindowTitle(u'PowerPoint Viewer Test')
+        self.setWindowTitle('PowerPoint Viewer Test')
 
-        ppt_label = QtGui.QLabel(u'Open PowerPoint file')
-        slide_label = QtGui.QLabel(u'Go to slide #')
-        self.pptEdit = QtGui.QLineEdit()
-        self.slideEdit = QtGui.QLineEdit()
-        x_label = QtGui.QLabel(u'X pos')
-        y_label = QtGui.QLabel(u'Y pos')
-        width_label = QtGui.QLabel(u'Width')
-        height_label = QtGui.QLabel(u'Height')
-        self.xEdit = QtGui.QLineEdit(u'100')
-        self.yEdit = QtGui.QLineEdit(u'100')
-        self.widthEdit = QtGui.QLineEdit(u'900')
-        self.heightEdit = QtGui.QLineEdit(u'700')
-        self.total = QtGui.QLabel()
-        ppt_btn = QtGui.QPushButton(u'Open')
-        ppt_dlg_btn = QtGui.QPushButton(u'...')
-        folder_label = QtGui.QLabel(u'Slide .bmp path')
-        self.folderEdit = QtGui.QLineEdit(u'slide')
-        slide_btn = QtGui.QPushButton(u'Go')
-        prev = QtGui.QPushButton(u'Prev')
-        next = QtGui.QPushButton(u'Next')
-        blank = QtGui.QPushButton(u'Blank')
-        unblank = QtGui.QPushButton(u'Unblank')
-        restart = QtGui.QPushButton(u'Restart')
-        close = QtGui.QPushButton(u'Close')
-        resume = QtGui.QPushButton(u'Resume')
-        stop = QtGui.QPushButton(u'Stop')
-        grid = QtGui.QGridLayout()
+        ppt_label = QtWidgets.QLabel('Open PowerPoint file')
+        slide_label = QtWidgets.QLabel('Go to slide #')
+        self.pptEdit = QtWidgets.QLineEdit()
+        self.slideEdit = QtWidgets.QLineEdit()
+        x_label = QtWidgets.QLabel('X pos')
+        y_label = QtWidgets.QLabel('Y pos')
+        width_label = QtWidgets.QLabel('Width')
+        height_label = QtWidgets.QLabel('Height')
+        self.xEdit = QtWidgets.QLineEdit('100')
+        self.yEdit = QtWidgets.QLineEdit('100')
+        self.widthEdit = QtWidgets.QLineEdit('900')
+        self.heightEdit = QtWidgets.QLineEdit('700')
+        self.total = QtWidgets.QLabel()
+        ppt_btn = QtWidgets.QPushButton('Open')
+        ppt_dlg_btn = QtWidgets.QPushButton('...')
+        folder_label = QtWidgets.QLabel('Slide .bmp path')
+        self.folderEdit = QtWidgets.QLineEdit('slide')
+        slide_btn = QtWidgets.QPushButton('Go')
+        prev = QtWidgets.QPushButton('Prev')
+        next = QtWidgets.QPushButton('Next')
+        blank = QtWidgets.QPushButton('Blank')
+        unblank = QtWidgets.QPushButton('Unblank')
+        restart = QtWidgets.QPushButton('Restart')
+        close = QtWidgets.QPushButton('Close')
+        resume = QtWidgets.QPushButton('Resume')
+        stop = QtWidgets.QPushButton('Stop')
+        grid = QtWidgets.QGridLayout()
         row = 0
         grid.addWidget(folder_label, 0, 0)
         grid.addWidget(self.folderEdit, 0, 1)
-        row = row + 1
+        row += 1
         grid.addWidget(x_label, row, 0)
         grid.addWidget(self.xEdit, row, 1)
         grid.addWidget(y_label, row, 2)
         grid.addWidget(self.yEdit, row, 3)
-        row = row + 1
+        row += 1
         grid.addWidget(width_label, row, 0)
         grid.addWidget(self.widthEdit, row, 1)
         grid.addWidget(height_label, row, 2)
         grid.addWidget(self.heightEdit, row, 3)
-        row = row + 1
+        row += 1
         grid.addWidget(ppt_label, row, 0)
         grid.addWidget(self.pptEdit, row, 1)
         grid.addWidget(ppt_dlg_btn, row, 2)
         grid.addWidget(ppt_btn, row, 3)
-        row = row + 1
+        row += 1
         grid.addWidget(slide_label, row, 0)
         grid.addWidget(self.slideEdit, row, 1)
         grid.addWidget(slide_btn, row, 2)
-        row = row + 1
+        row += 1
         grid.addWidget(prev, row, 0)
         grid.addWidget(next, row, 1)
-        row = row + 1
+        row += 1
         grid.addWidget(blank, row, 0)
         grid.addWidget(unblank, row, 1)
-        row = row + 1
+        row += 1
         grid.addWidget(restart, row, 0)
         grid.addWidget(close, row, 1)
-        row = row + 1
+        row += 1
         grid.addWidget(stop, row, 0)
         grid.addWidget(resume, row, 1)
-        self.connect(ppt_btn, QtCore.SIGNAL(u'clicked()'), self.openClick)
-        self.connect(ppt_dlg_btn, QtCore.SIGNAL(u'clicked()'), self.openDialog)
-        self.connect(slide_btn, QtCore.SIGNAL(u'clicked()'), self.gotoClick)
-        self.connect(prev, QtCore.SIGNAL(u'clicked()'), self.prevClick)
-        self.connect(next, QtCore.SIGNAL(u'clicked()'), self.nextClick)
-        self.connect(blank, QtCore.SIGNAL(u'clicked()'), self.blankClick)
-        self.connect(unblank, QtCore.SIGNAL(u'clicked()'), self.unblankClick)
-        self.connect(restart, QtCore.SIGNAL(u'clicked()'), self.restartClick)
-        self.connect(close, QtCore.SIGNAL(u'clicked()'), self.closeClick)
-        self.connect(stop, QtCore.SIGNAL(u'clicked()'), self.stopClick)
-        self.connect(resume, QtCore.SIGNAL(u'clicked()'), self.resumeClick)
+        ppt_btn.clicked.connect(self.openClick)
+        ppt_dlg_btn.clicked.connect(self.openDialog)
+        slide_btn.clicked.connect(self.gotoClick)
+        prev.clicked.connect(self.prevClick)
+        next.clicked.connect(self.nextClick)
+        blank.clicked.connect(self.blankClick)
+        unblank.clicked.connect(self.unblankClick)
+        restart.clicked.connect(self.restartClick)
+        close.clicked.connect(self.closeClick)
+        stop.clicked.connect(self.stopClick)
+        resume.clicked.connect(self.resumeClick)
         self.setLayout(grid)
         self.resize(300, 150)
 
@@ -169,47 +163,46 @@ class PPTViewer(QtGui.QWidget):
         app.processEvents()
 
     def openClick(self):
-        oldid = self.pptid;
+        oldid = self.pptid
         rect = RECT(int(self.xEdit.text()), int(self.yEdit.text()),
-            int(self.widthEdit.text()), int(self.heightEdit.text()))
-        filename = str(self.pptEdit.text().replace(u'/', u'\\'))
-        folder = str(self.folderEdit.text().replace(u'/', u'\\'))
-        print filename, folder
+                    int(self.widthEdit.text()), int(self.heightEdit.text()))
+        filename = str(self.pptEdit.text().replace('/', '\\'))
+        folder = str(self.folderEdit.text().replace('/', '\\'))
+        print(filename, folder)
         self.pptid = self.pptdll.OpenPPT(filename, None, rect, folder)
-        print u'id: ' + unicode(self.pptid)
+        print('id: ' + str(self.pptid))
         if oldid >= 0:
-            self.pptdll.ClosePPT(oldid);
+            self.pptdll.ClosePPT(oldid)
         slides = self.pptdll.GetSlideCount(self.pptid)
-        print u'slidecount: ' + unicode(slides)
+        print('slidecount: ' + str(slides))
         self.total.setNum(self.pptdll.GetSlideCount(self.pptid))
         self.updateCurrSlide()
 
     def updateCurrSlide(self):
         if self.pptid < 0:
             return
-        slide = unicode(self.pptdll.GetCurrentSlide(self.pptid))
-        print u'currslide: ' + slide
+        slide = str(self.pptdll.GetCurrentSlide(self.pptid))
+        print('currslide: ' + slide)
         self.slideEdit.setText(slide)
         app.processEvents()
 
     def gotoClick(self):
         if self.pptid < 0:
             return
-        print self.slideEdit.text()
+        print(self.slideEdit.text())
         self.pptdll.GotoSlide(self.pptid, int(self.slideEdit.text()))
         self.updateCurrSlide()
         app.processEvents()
 
     def openDialog(self):
-        self.pptEdit.setText(QtGui.QFileDialog.getOpenFileName(self,
-            u'Open file'))
+        self.pptEdit.setText(QtWidgets.QFileDialog.getOpenFileName(self, 'Open file')[0])
 
 if __name__ == '__main__':
     pptdll = cdll.LoadLibrary(r'pptviewlib.dll')
     pptdll.SetDebug(1)
-    print u'Begin...'
-    app = QtGui.QApplication(sys.argv)
+    print('Begin...')
+    app = QtWidgets.QApplication(sys.argv)
     window = PPTViewer()
     window.pptdll = pptdll
     window.show()
-    sys.exit(app.exec_())
+    sys.exit(app.exec())
