@@ -1,17 +1,10 @@
 # -*- coding: utf-8 -*-
-# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4
+# vim: autoindent shiftwidth=4 expandtab textwidth=120 tabstop=4 softtabstop=4
 
 ###############################################################################
 # OpenLP - Open Source Lyrics Projection                                      #
 # --------------------------------------------------------------------------- #
-# Copyright (c) 2008-2014 Raoul Snyman                                        #
-# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
-# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
-# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
-# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
-# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
-# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
-# Frode Woldsund, Martin Zibricky                                             #
+# Copyright (c) 2008-2016 OpenLP Developers                                   #
 # --------------------------------------------------------------------------- #
 # This program is free software; you can redistribute it and/or modify it     #
 # under the terms of the GNU General Public License as published by the Free  #
@@ -31,20 +24,18 @@ import logging
 import os
 import shutil
 
-from PyQt4 import QtCore
+from PyQt5 import QtCore
 
-from openlp.core.lib import Receiver, check_directory_exists, create_thumb, \
-    validate_thumb
-from openlp.core.lib.settings import Settings
-from openlp.core.utils import AppLocation
+from openlp.core.common import Registry, AppLocation, Settings, check_directory_exists, md5_hash
+from openlp.core.lib import create_thumb, validate_thumb
 
 log = logging.getLogger(__name__)
 
+
 class PresentationDocument(object):
     """
-    Base class for presentation documents to inherit from.
-    Loads and closes the presentation as well as triggering the correct
-    activities based on the users input
+    Base class for presentation documents to inherit from. Loads and closes the presentation as well as triggering the
+    correct activities based on the users input
 
     **Hook Functions**
 
@@ -98,9 +89,15 @@ class PresentationDocument(object):
         """
         Constructor for the PresentationController class
         """
-        self.slidenumber = 0
         self.controller = controller
-        self.filepath = name
+        self._setup(name)
+
+    def _setup(self, name):
+        """
+        Run some initial setup. This method is separate from __init__ in order to mock it out in tests.
+        """
+        self.slide_number = 0
+        self.file_path = name
         check_directory_exists(self.get_thumbnail_folder())
 
     def load_presentation(self):
@@ -118,45 +115,53 @@ class PresentationDocument(object):
         a file, e.g. thumbnails
         """
         try:
-            shutil.rmtree(self.get_thumbnail_folder())
-            shutil.rmtree(self.get_temp_folder())
+            if os.path.exists(self.get_thumbnail_folder()):
+                shutil.rmtree(self.get_thumbnail_folder())
+            if os.path.exists(self.get_temp_folder()):
+                shutil.rmtree(self.get_temp_folder())
         except OSError:
-            log.exception(u'Failed to delete presentation controller files')
+            log.exception('Failed to delete presentation controller files')
 
     def get_file_name(self):
         """
-        Return just the filename of the presention, without the directory
+        Return just the filename of the presentation, without the directory
         """
-        return os.path.split(self.filepath)[1]
+        return os.path.split(self.file_path)[1]
 
     def get_thumbnail_folder(self):
         """
         The location where thumbnail images will be stored
         """
-        return os.path.join(
-            self.controller.thumbnail_folder, self.get_file_name())
+        # TODO: If statement can be removed when the upgrade path from 2.0.x to 2.2.x is no longer needed
+        if Settings().value('presentations/thumbnail_scheme') == 'md5':
+            folder = md5_hash(self.file_path.encode('utf-8'))
+        else:
+            folder = self.get_file_name()
+        return os.path.join(self.controller.thumbnail_folder, folder)
 
     def get_temp_folder(self):
         """
         The location where thumbnail images will be stored
         """
-        return os.path.join(
-            self.controller.temp_folder, self.get_file_name())
+        # TODO: If statement can be removed when the upgrade path from 2.0.x to 2.2.x is no longer needed
+        if Settings().value('presentations/thumbnail_scheme') == 'md5':
+            folder = md5_hash(self.file_path.encode('utf-8'))
+        else:
+            folder = folder = self.get_file_name()
+        return os.path.join(self.controller.temp_folder, folder)
 
     def check_thumbnails(self):
         """
-        Returns ``True`` if the thumbnail images exist and are more recent than
-        the powerpoint file.
+        Returns ``True`` if the thumbnail images exist and are more recent than the powerpoint file.
         """
-        lastimage = self.get_thumbnail_path(self.get_slide_count(), True)
-        if not (lastimage and os.path.isfile(lastimage)):
+        last_image = self.get_thumbnail_path(self.get_slide_count(), True)
+        if not (last_image and os.path.isfile(last_image)):
             return False
-        return validate_thumb(self.filepath, lastimage)
+        return validate_thumb(self.file_path, last_image)
 
     def close_presentation(self):
         """
-        Close presentation and clean up objects
-        Triggered by new object being added to SlideController
+        Close presentation and clean up objects. Triggered by new object being added to SlideController
         """
         self.controller.close_presentation()
 
@@ -180,7 +185,7 @@ class PresentationDocument(object):
 
     def unblank_screen(self):
         """
-        Unblanks (restores) the presentationn
+        Unblanks (restores) the presentation
         """
         pass
 
@@ -218,15 +223,14 @@ class PresentationDocument(object):
         """
         Jumps directly to the requested slide.
 
-        ``slide_no``
-            The slide to jump to, starting at 1
+        :param slide_no: The slide to jump to, starting at 1
         """
         pass
 
     def next_step(self):
         """
-        Triggers the next effect of slide on the running presentation
-        This might be the next animation on the current slide, or the next slide
+        Triggers the next effect of slide on the running presentation. This might be the next animation on the current
+        slide, or the next slide
         """
         pass
 
@@ -238,8 +242,7 @@ class PresentationDocument(object):
 
     def convert_thumbnail(self, file, idx):
         """
-        Convert the slide image the application made to a standard 320x240
-        .png image.
+        Convert the slide image the application made to a standard 320x240 .png image.
         """
         if self.check_thumbnails():
             return
@@ -251,11 +254,10 @@ class PresentationDocument(object):
         """
         Returns an image path containing a preview for the requested slide
 
-        ``slide_no``
-            The slide an image is required for, starting at 1
+        :param slide_no: The slide an image is required for, starting at 1
+        :param check_exists:
         """
-        path = os.path.join(self.get_thumbnail_folder(),
-            self.controller.thumbnail_prefix + unicode(slide_no) + u'.png')
+        path = os.path.join(self.get_thumbnail_folder(), self.controller.thumbnail_prefix + str(slide_no) + '.png')
         if os.path.isfile(path) or not check_exists:
             return path
         else:
@@ -269,22 +271,20 @@ class PresentationDocument(object):
             return
         if not hide_mode:
             current = self.get_slide_number()
-            if current == self.slidenumber:
+            if current == self.slide_number:
                 return
-            self.slidenumber = current
+            self.slide_number = current
         if is_live:
-            prefix = u'live'
+            prefix = 'live'
         else:
-            prefix = u'preview'
-        Receiver.send_message(u'slidecontroller_%s_change' % prefix,
-            self.slidenumber - 1)
+            prefix = 'preview'
+        Registry().execute('slidecontroller_%s_change' % prefix, self.slide_number - 1)
 
     def get_slide_text(self, slide_no):
         """
         Returns the text on the slide
 
-        ``slide_no``
-        The slide the text is required for, starting at 1
+        :param slide_no: The slide the text is required for, starting at 1
         """
         return ''
 
@@ -292,25 +292,64 @@ class PresentationDocument(object):
         """
         Returns the text on the slide
 
-        ``slide_no``
-        The slide the notes are required for, starting at 1
+        :param slide_no: The slide the text is required for, starting at 1
         """
         return ''
 
+    def get_titles_and_notes(self):
+        """
+        Reads the titles from the titles file and
+        the notes files and returns the content in two lists
+        """
+        titles = []
+        notes = []
+        titles_file = os.path.join(self.get_thumbnail_folder(), 'titles.txt')
+        if os.path.exists(titles_file):
+            try:
+                with open(titles_file, encoding='utf-8') as fi:
+                    titles = fi.read().splitlines()
+            except:
+                log.exception('Failed to open/read existing titles file')
+                titles = []
+        for slide_no, title in enumerate(titles, 1):
+            notes_file = os.path.join(self.get_thumbnail_folder(), 'slideNotes%d.txt' % slide_no)
+            note = ''
+            if os.path.exists(notes_file):
+                try:
+                    with open(notes_file, encoding='utf-8') as fn:
+                        note = fn.read()
+                except:
+                    log.exception('Failed to open/read notes file')
+                    note = ''
+            notes.append(note)
+        return titles, notes
+
+    def save_titles_and_notes(self, titles, notes):
+        """
+        Performs the actual persisting of titles to the titles.txt
+        and notes to the slideNote%.txt
+        """
+        if titles:
+            titles_file = os.path.join(self.get_thumbnail_folder(), 'titles.txt')
+            with open(titles_file, mode='wt', encoding='utf-8') as fo:
+                fo.writelines(titles)
+        if notes:
+            for slide_no, note in enumerate(notes, 1):
+                notes_file = os.path.join(self.get_thumbnail_folder(), 'slideNotes%d.txt' % slide_no)
+                with open(notes_file, mode='wt', encoding='utf-8') as fn:
+                    fn.write(note)
+
 
 class PresentationController(object):
     """
-    This class is used to control interactions with presentation applications
-    by creating a runtime environment. This is a base class for presentation
-    controllers to inherit from.
+    This class is used to control interactions with presentation applications by creating a runtime environment.
+    This is a base class for presentation controllers to inherit from.
 
-    To create a new controller, take a copy of this file and name it so it ends
-    with ``controller.py``, i.e. ``foobarcontroller.py``. Make sure it inherits
-    :class:`~openlp.plugins.presentations.lib.presentationcontroller.PresentationController`,
-    and then fill in the blanks. If possible try to make sure it loads on all
-    platforms, usually by using :mod:``os.name`` checks, although
-    ``__init__``, ``check_available`` and ``presentation_deleted`` should
-    always be implemented.
+    To create a new controller, take a copy of this file and name it so it ends with ``controller.py``, i.e.
+    ``foobarcontroller.py``. Make sure it inherits
+    :class:`~openlp.plugins.presentations.lib.presentationcontroller.PresentationController`, and then fill in the
+    blanks. If possible try to make sure it loads on all platforms, usually by using :mod:``os.name`` checks, although
+    ``__init__``, ``check_available`` and ``presentation_deleted`` should always be implemented.
 
     See :class:`~openlp.plugins.presentations.lib.impresscontroller.ImpressController`,
     :class:`~openlp.plugins.presentations.lib.powerpointcontroller.PowerpointController` or
@@ -320,72 +359,64 @@ class PresentationController(object):
     **Basic Attributes**
 
     ``name``
-        The name that appears in the options and the media manager
+        The name that appears in the options and the media manager.
 
     ``enabled``
-        The controller is enabled
+        The controller is enabled.
 
     ``available``
-        The controller is available on this machine. Set by init via
-        call to check_available
+        The controller is available on this machine. Set by init via call to check_available.
 
     ``plugin``
-        The presentationplugin object
+        The presentationplugin object.
 
     ``supports``
-        The primary native file types this application supports
+        The primary native file types this application supports.
 
-    ``alsosupports``
-        Other file types the application can import, although not necessarily
-        the first choice due to potential incompatibilities
+    ``also_supports``
+        Other file types the application can import, although not necessarily the first choice due to potential
+        incompatibilities.
 
     **Hook Functions**
 
     ``kill()``
-        Called at system exit to clean up any running presentations
+        Called at system exit to clean up any running presentations.
 
     ``check_available()``
-        Returns True if presentation application is installed/can run on this
-        machine
+        Returns True if presentation application is installed/can run on this machine.
 
     ``presentation_deleted()``
-        Deletes presentation specific files, e.g. thumbnails
+        Deletes presentation specific files, e.g. thumbnails.
 
     """
-    log.info(u'PresentationController loaded')
+    log.info('PresentationController loaded')
 
-    def __init__(self, plugin=None, name=u'PresentationController',
-        document_class=PresentationDocument):
+    def __init__(self, plugin=None, name='PresentationController', document_class=PresentationDocument):
         """
-        This is the constructor for the presentationcontroller object. This
-        provides an easy way for descendent plugins to populate common data.
-        This method *must* be overridden, like so::
+        This is the constructor for the presentationcontroller object. This provides an easy way for descendent plugins
+
+        to populate common data. This method *must* be overridden, like so::
 
             class MyPresentationController(PresentationController):
                 def __init__(self, plugin):
                     PresentationController.__init(
-                        self, plugin, u'My Presenter App')
+                        self, plugin, 'My Presenter App')
 
-        ``plugin``
-            Defaults to *None*. The presentationplugin object
-
-        ``name``
-            Name of the application, to appear in the application
+        :param plugin:  Defaults to *None*. The presentationplugin object
+        :param name: Name of the application, to appear in the application
+        :param document_class:
         """
         self.supports = []
-        self.alsosupports = []
+        self.also_supports = []
         self.docs = []
         self.plugin = plugin
         self.name = name
         self.document_class = document_class
-        self.settings_section = self.plugin.settingsSection
+        self.settings_section = self.plugin.settings_section
         self.available = None
-        self.temp_folder = os.path.join(
-            AppLocation.get_section_data_path(self.settings_section), name)
-        self.thumbnail_folder = os.path.join(
-            AppLocation.get_section_data_path(self.settings_section),
-            u'thumbnails')
-        self.thumbnail_prefix = u'slide'
+        self.temp_folder = os.path.join(AppLocation.get_section_data_path(self.settings_section), name)
+        self.thumbnail_folder = os.path.join(AppLocation.get_section_data_path(self.settings_section), 'thumbnails')
+        self.thumbnail_prefix = 'slide'
         check_directory_exists(self.thumbnail_folder)
         check_directory_exists(self.temp_folder)
 
@@ -393,10 +424,7 @@ class PresentationController(object):
         """
         Return whether the controller is currently enabled
         """
-        if Settings().value(
-            self.settings_section + u'/' + self.name,
-            QtCore.QVariant(QtCore.Qt.Checked)).toInt()[0] == \
-                QtCore.Qt.Checked:
+        if Settings().value(self.settings_section + '/' + self.name) == QtCore.Qt.Checked:
             return self.is_available()
         else:
             return False
@@ -408,28 +436,26 @@ class PresentationController(object):
 
     def check_available(self):
         """
-        Presentation app is able to run on this machine
+        Presentation app is able to run on this machine.
         """
         return False
 
     def start_process(self):
         """
-        Loads a running version of the presentation application in the
-        background.
+        Loads a running version of the presentation application in the background.
         """
         pass
 
     def kill(self):
         """
-        Called at system exit to clean up any running presentations and
-        close the application
+        Called at system exit to clean up any running presentations and close the application.
         """
-        log.debug(u'Kill')
+        log.debug('Kill')
         self.close_presentation()
 
     def add_document(self, name):
         """
-        Called when a new presentation document is opened
+        Called when a new presentation document is opened.
         """
         document = self.document_class(self, name)
         self.docs.append(document)
@@ -437,9 +463,9 @@ class PresentationController(object):
 
     def remove_doc(self, doc=None):
         """
-        Called to remove an open document from the collection
+        Called to remove an open document from the collection.
         """
-        log.debug(u'remove_doc Presentation')
+        log.debug('remove_doc Presentation')
         if doc is None:
             return
         if doc in self.docs:
@@ -447,3 +473,12 @@ class PresentationController(object):
 
     def close_presentation(self):
         pass
+
+
+class TextType(object):
+    """
+    Type Enumeration for Types of Text to request
+    """
+    Title = 0
+    SlideText = 1
+    Notes = 2
